% In this specification, we define H1 * [f0t, f4t, f8t]' as factors, and assume
% H1 * yt priced exactly. See hw4_20110813
clear
clc
addpath(genpath('functions_estimation'))
addpath(genpath('matrices'))

load panel_close_weekly_matu_9001to1106

%=========================================================
%If you run this section, it produces estimates in Table 2
tt = [1,180];
load parameters_base_90to04
%=========================================================

%=========================================================
%If you run this section, it produces estimates in Table 3
% tt = [181,256];
% load parameters_base_05to11
%=========================================================

%% calculate y1 and y2 for each j
%P1 = [1/3, 1/3, 1/3; -1, 0, 1];
P1 = [0 (1/2) (1/2); 0 -1 1];
P2 = [1 0 0];
% j = 1 
y11 = y1_matu(tt(1):tt(2),:)*P1';% notation yj,i, where j = 1,2,3,4 week, and i = 1,2 % NOTE, the notation for A and B reverse order!
y12 = y1_matu(tt(1):tt(2),:)*P2';
% j = 2 
y21 = y2_matu(tt(1):tt(2),:)*P1';
y22 = y2_matu(tt(1):tt(2),:)*P2';
% j = 3 
y31 = y3_matu(tt(1):tt(2),:)*P1';
y32 = y3_matu(tt(1):tt(2),:)*P2';
% j = 4 
y41 = y4_matu(tt(1):tt(2),:)*P1';
y42 = y4_matu(tt(1):tt(2),:)*P2';


%% OLS regressions
% first, estimate the y1 equations to solve phi, Phi and Omega, and
% calculate information matrix
% j = 1
[phi1,Phi1,Omega1] = ols(y11(2:end,:),y41(1:end-1,:),1);
[H_11, Hv_11] = information_matrix(y41(1:end-1,:),Omega1,1);
% j = 2
[phi2,Phi2,Omega2] = ols(y21(2:end,:),y11(2:end,:),1);
[H_12, Hv_12] = information_matrix(y11(2:end,:),Omega2,1);
% j = 3
[phi3,Phi3,Omega3] = ols(y31(2:end,:),y21(2:end,:),1);
[H_13, Hv_13] = information_matrix(y21(2:end,:),Omega3,1);
% j = 4
[phi4,Phi4,Omega4] = ols(y41(2:end,:),y31(2:end,:),1);
[H_14, Hv_14] = information_matrix(y31(2:end,:),Omega4,1);

% second, estimate the y2 equation to solve for gamma, Gamma, and
% calculate information matrix
% j = 1
[gamma1, Gamma1,Sigma_e1,e1] = ols(y12(2:end,:),y11(2:end,:),1);
% j = 2
[gamma2, Gamma2,Sigma_e2,e2] = ols(y22(2:end,:),y21(2:end,:),1);
% j = 3
[gamma3, Gamma3,Sigma_e3,e3] = ols(y32(2:end,:),y31(2:end,:),1);
% j = 4
[gamma4, Gamma4,Sigma_e4,e4] = ols(y42(2:end,:),y41(2:end,:),1);

[H_21] = information_matrix(y11(2:end,:),Sigma_e1,1);
[H_22] = information_matrix(y21(2:end,:),Sigma_e2,1);
[H_23] = information_matrix(y31(2:end,:),Sigma_e3,1);
[H_24] = information_matrix(y41(2:end,:),Sigma_e4,1);

Sigma_e1_std = sqrt(Sigma_e1/(2*diff(tt)));
Sigma_e2_std = sqrt(Sigma_e2/(2*diff(tt)));
Sigma_e3_std = sqrt(Sigma_e3/(2*diff(tt)));
Sigma_e4_std = sqrt(Sigma_e4/(2*diff(tt)));
%% MCS estimation
options  =  optimset('fminunc');
options  =  optimset(options , 'TolFun'      , 1e-20);
options  =  optimset(options , 'TolX'      , 1e-15);
options  =  optimset(options , 'Display'     , 'iter');
options  =  optimset(options , 'Diagnostics' , 'on');
options  =  optimset(options , 'LargeScale'  , 'off');
options  =  optimset(options , 'MaxIter' , 5000) ;
options  =  optimset(options , 'MaxFunEvals' , 50000) ;

% initial condition
%{
%c = (phi1+phi2+phi3+phi4)/4;
c = zeros(2,1);
%rho = (Phi1+Phi2+Phi3+Phi4)/4;
%rho = (Phi1+Phi2+Phi3+Phi4)/4 + diag(randn(2,1))/20;
rho = diag(rand(2,1)*0.2+0.85);    
Sigma = chol((Omega1+Omega2+Omega3+Omega4)/4)';
%xi = eig(rho);
xi = rand(2,1)*0.2+0.85;

%alpha_0 = gamma3;
alpha_0 = 0;
%}


parameters = para_ind2vec_HW4(xi,alpha_0,c, rho,Sigma);

for j = 1:3
    initial = parameters;
    [parameters,F] =  fminunc('MCS_HW4',initial, options,...
        phi1,Phi1,Omega1,phi2,Phi2,Omega2,phi3,Phi3,Omega3,phi4,Phi4,Omega4,...
        gamma1, Gamma1,gamma2, Gamma2,gamma3, Gamma3,gamma4, Gamma4,...
        H_11, Hv_11,H_12, Hv_12,H_13, Hv_13,H_14, Hv_14,...
        H_21, H_22, H_23,H_24,P1,P2);
end
[xi,alpha_0,c, rho,Sigma] = para_vec2ind_HW4(parameters);

%save('parameters_base_90to11', 'xi','alpha_0','c', 'rho','Sigma')
%save('parameters_base_90to04', 'xi','alpha_0','c', 'rho','Sigma')
%save('parameters_base_05to11', 'xi','alpha_0','c', 'rho','Sigma')

[F,A1,A2,A3,A4,B1,B2,B3,B4,cQ,rhoQ,beta_0,alpha,beta_transpose] = MCS_HW4(parameters,phi1,Phi1,Omega1,phi2,Phi2,Omega2,phi3,Phi3,Omega3,phi4,Phi4,Omega4,...
    gamma1, Gamma1,gamma2, Gamma2,gamma3, Gamma3,gamma4, Gamma4,...
    H_11, Hv_11,H_12, Hv_12,H_13, Hv_13,H_14, Hv_14,...
    H_21, H_22, H_23,H_24,P1,P2);


%save('AB_base_newP1','alpha', 'beta_transpose')
%% calculate asymptotic standard errors
lam = c-cQ;
LAM = rho - rhoQ;


addpath(genpath('functions_asymptoticSTD'))

R = blkdiag(H_11, Hv_11,H_12, Hv_12,H_13, Hv_13,H_14, Hv_14,H_21,H_22,H_23,H_24);
GAMMA = numgrad('papa_str2redu_vec',parameters,P1,P2);
variance = inv(GAMMA'*R*GAMMA);
para_std = sqrt(diag(variance));
[xi_std,alpha_0_std,c_std, rho_std,Sigma_std] = para_vec2ind_HW4(para_std);


parameters2 = para_ind2vec_HW4(xi,alpha_0,lam, LAM,Sigma);
GAMMA = numgrad('papa_str2redu_vec_lam',parameters2,P1,P2);
variance = inv(GAMMA'*R*GAMMA);
para_std = sqrt(diag(variance));
[xi_std,alpha_0_std,lam_std, LAM_std,Sigma_std] = para_vec2ind_HW4(para_std);

%% back out factors
T = length(y11);
x_1 = (y11 - repmat((P1*A1)',T,1))/(P1*B1)';
x_2 = (y21 - repmat((P1*A2)',T,1))/(P1*B2)';
x_3 = (y31 - repmat((P1*A3)',T,1))/(P1*B3)';
x_4 = (y41 - repmat((P1*A4)',T,1))/(P1*B4)';


% combine factors into weekly frequency
x = zeros(4*T,2);
for i = 1:T
    x((i-1)*4+1:i*4,:) = [x_1(i,:);x_2(i,:);x_3(i,:);x_4(i,:)];
end


%lam+ LAM*mean(x)';
x_bar = mean(x)';

lam_std + LAM_std*x_bar;

%parameters3 = para_ind2vec_HW4(zeros(2,1),0,(1:2)',reshape(3:6,2,2) ,zeros(2));
temp = zeros(12,1);
temp(4) = 1/10;
temp(6) = x_bar(1);
temp(8) = x_bar(2);
sqrt(temp'*variance*temp);

temp = zeros(12,1);
temp(5) = 1/10;
temp(7) = x_bar(1);
temp(9) = x_bar(2);
sqrt(temp'*variance*temp);

risk_price = repmat(lam',4*T,1)+x*LAM';

risk_price_std = zeros(size(risk_price));
for i = 1:4*T
    temp = zeros(12,1);
    temp(4) = 1/10;
    temp(6) = x(i,1);
    temp(8) = x(i,2);
    risk_price_std(i,1) = sqrt(temp'*variance*temp);

    temp = zeros(12,1);
    temp(5) = 1/10;
    temp(7) = x(i,1);
    temp(9) = x(i,2);
    risk_price_std(i,2) = sqrt(temp'*variance*temp);
end

%save risk_price_2 risk_price risk_price_std

% risk premium
[alpha_,beta_transpose_] = ab_baseline(c, rho, Sigma, alpha_0,beta_0);
%rp = alpha(5)+x*(beta_transpose(5,:))' - alpha_(5)- x*(beta_transpose_(5,:))';
rp = alpha(9)+x*(beta_transpose(9,:))' - alpha_(9)- x*(beta_transpose_(9,:))';

%save rp1 rp


%% plot price of risk
load risk_price_1
risk_price1 = risk_price;
risk_price_std1 = risk_price_std;
load risk_price_2
risk_price = [risk_price1;risk_price];
risk_price_std = [risk_price_std1;risk_price_std];

T = 256;
for i = 1:T
    date_weekly((i-1)*4+1:i*4,:) = [date1(i,:);date2(i,:);date3(i,:);date4(i,:)];
end
[date_u,index] = unique(floor(date_weekly/1e4),'first');
index(1) = [];

subplot(2,1,1), plot(datenum(num2str(date_weekly(1:4*T)),'yyyymmdd'),risk_price(:,1),'linewidth',2)
set(gca,'XTick',datenum(num2str(date_weekly(index)),'yyyymmdd'))
datetick('x','yyyy','keeplimits')
hold on
plot(datenum(num2str(date_weekly(1:4*T)),'yyyymmdd'),[risk_price(:,1)-2*risk_price_std(:,1),risk_price(:,1)+2*risk_price_std(:,1)],'g--')
title('Level risk price')
plot(datenum(num2str(20050101),'yyyymmdd'),linspace(-0.08,0.06,1000),'r')
hline = refline(0); 
set(hline,'Color',[0.6 0.6 0.6])
axis([datenum(num2str(19900101),'yyyymmdd'),datenum(num2str(20110630),'yyyymmdd'),-inf,inf])



subplot(2,1,2), plot(datenum(num2str(date_weekly(1:4*T)),'yyyymmdd'),risk_price(:,2),'linewidth',2)
set(gca,'XTick',datenum(num2str(date_weekly(index)),'yyyymmdd'))
datetick('x','yyyy','keeplimits')
hold on
plot(datenum(num2str(date_weekly(1:4*T)),'yyyymmdd'),[risk_price(:,2)-2*risk_price_std(:,2),risk_price(:,2)+2*risk_price_std(:,2)],'g--')
title('Slope risk price')
plot(datenum(num2str(20050101),'yyyymmdd'),linspace(-6e-3,6e-3,1000),'r')
hline = refline(0); 
set(hline,'Color',[0.6 0.6 0.6])
axis([datenum(num2str(19900101),'yyyymmdd'),datenum(num2str(20110630),'yyyymmdd'),-inf,inf])


%% estimate term premium
figure
load rp1
rp1 = rp;
load rp2
rp = [rp1;rp];

plot(datenum(num2str(date_weekly(1:4*T)),'yyyymmdd'),-rp)
set(gca,'XTick',datenum(num2str(date_weekly(index)),'yyyymmdd'))
datetick('x','yyyy','keeplimits')
hold on
hline = refline(0); 
set(hline,'Color',[0.6 0.6 0.6])
%title('4-week risk premium')
title('8-week risk premium')

plot(datenum(num2str(20050101),'yyyymmdd'),linspace(-0.18,0.18,1000),'r')