function [date_weekly,data_weekly,contracts,week_index] = daily_to_pseudoweekly_expiry(date_daily,contracts_all, data_daily,m_begin, m_end)
% function daily_to_pseudoweekly_expiry converts (business) daily data into pseudo weekly data, defined as last biz days of previous month, 5th biz days, 10th biz days and ending on expiry day.
% INPUTS:   date_daily (T*1): numerical date in daily frequency (for business
%               days), in the format of YYYYMMDD
%           contracts_all (1*N): a row vector containing contract names in
%               the format of YYYYMM 
%           data_daily: (T*N): (i,j) element corresponds to close price of contracts_all(j) observed on date_daily(i)
%           m_begin: how many months to take out at the beginning
%               = 0 if and only if the first contract expires on the 1st
%               month, and we have complete observation on the 1st month.
%               = 2 if the first contract expires on the 3rd month
%           m_end: how many months to take out at the end
%               = 0


YYYYMM = floor(date_daily/100);
YYYYMM = unique(YYYYMM);
YYYYMM(YYYYMM<m_begin) = [];
YYYYMM(YYYYMM>m_end) = [];

T = length(YYYYMM);
N = size(data_daily,2);
date_weekly = zeros(4*T,1);
week_index = repmat([1:4]',T,1);
data_weekly = zeros(4*T,N);

for i = 1:T
    index_imonth = floor(date_daily/100) == YYYYMM(i);
    index_imonthminus1 = min(find(index_imonth==1))-1;
    temp_date = date_daily(index_imonth);
    temp_data = data_daily(index_imonth,:);
    temp_data2 = temp_data;
    temp_data2(:,isnan(temp_data(1,:))) = [];
    index_ex = max(find(~isnan(temp_data2(:,1))));
    date_weekly((i-1)*4+1:4*i) = [date_daily(index_imonthminus1);temp_date([5,10,index_ex])];
    data_weekly((i-1)*4+1:4*i,:) = [data_daily(index_imonthminus1,:);temp_data([5,10,index_ex],:)];
end

data_weekly(:,contracts_all < month_add(round(date_weekly(1)/100), 2)) = [];
contracts_all(contracts_all < month_add(round(date_weekly(1)/100), 2)) = [];
data_weekly(:,contracts_all > month_add(round(date_weekly(end)/100), 1)) = [];
contracts_all(contracts_all > month_add(round(date_weekly(end)/100), 1)) = [];
contracts = contracts_all;
