function [penalty,phi1,Phi1,Omega1,phi2,Phi2,Omega2,phi3,Phi3,Omega3,phi4,Phi4,Omega4,...
    gamma1, Gamma1,gamma2, Gamma2,gamma3, Gamma3,gamma4, Gamma4,A1,A2,A3,A4,B1,B2,B3,B4,cQ,rhoQ,beta_0,alpha,beta_transpose]...
    = papa_str2redu(xi,Sigma, c, rho, alpha_0,H1,H2)
% convert_papa_str2redu imposes cross section restrictions and converts 
% structure parameters into reduced form parameters

%H1 = [1/3, 1/3, 1/3; -1, 0, 1];
%H2 = [1, -2, 1];
%=========================create sigma*sigma'==============================
Sigma_sq = Sigma*Sigma';

%====================CALCULATE rhoQ, beta_0================================
K_xi = [xi.^0,xi.^4,xi.^8];
rhoQ = ((K_xi*H1')\diag(xi)*(K_xi*H1'))';
beta_0 = (K_xi*H1')\ones(2,1);

%==========================calculate beta==================================
beta_transpose = zeros(12,2);  % ith row of beta_transpose is beta_i-1'
beta_transpose(1,:) = beta_0';
for i = 2:12
    beta_transpose(i,:) = beta_transpose(i-1,:)*rhoQ;
end

%=====================create beta'*sigma*sigma'*beta=======================
beta_sigsig_beta = zeros(12,1);
for i = 1:12
    beta_sigsig_beta(i) = beta_transpose(i,:)*Sigma_sq*beta_transpose(i,:)';
end

%============================calculate cQ==================================
cQ = -(H1*[zeros(1,2);sum(beta_transpose(1:4,:));sum(beta_transpose(1:8,:))])\...
    (H1*[alpha_0;alpha_0+0.5*sum(beta_sigsig_beta(1:4));alpha_0+0.5*sum(beta_sigsig_beta(1:8))]);

%==========================calculate alpha=================================
alpha = zeros(12,1);
alpha(1) = alpha_0;
for i = 2:12
    alpha(i) = alpha(i-1)+beta_transpose(i-1,:)*cQ + 0.5 * beta_sigsig_beta(i-1);
end

%==================convert to reduced form parameters======================
B1 = [beta_transpose(3+1,:);beta_transpose(7+1,:);beta_transpose(11+1,:)]; 
B2 = [beta_transpose(2+1,:);beta_transpose(6+1,:);beta_transpose(10+1,:)];
B3 = [beta_transpose(1+1,:);beta_transpose(5+1,:);beta_transpose(9+1,:)];
B4 = [beta_transpose(0+1,:);beta_transpose(4+1,:);beta_transpose(8+1,:)];


if min([rcond(H1*B1), rcond(H1*B2),rcond(H1*B3),rcond(H1*B4)])<1e-15
    penalty = 1e50;
    phi1=[];Phi1=[];Omega1=[];
    phi2=[];Phi2=[];Omega2=[];
    phi3=[];Phi3=[];Omega3=[];
    phi4=[];Phi4=[];Omega4=[];
    gamma1=[]; Gamma1=[];
    gamma2=[]; Gamma2=[];
    gamma3=[]; Gamma3=[];
else
    penalty = 0;
    A1 = [alpha(3+1);alpha(7+1);alpha(11+1)]; 
    A2 = [alpha(2+1);alpha(6+1);alpha(10+1)];
    A3 = [alpha(1+1);alpha(5+1);alpha(9+1)];
    A4 = [alpha(0+1);alpha(4+1);alpha(8+1)];
    
    Omega1 = H1*B1*Sigma_sq*(H1*B1)';
    Omega2 = H1*B2*Sigma_sq*(H1*B2)';
    Omega3 = H1*B3*Sigma_sq*(H1*B3)';
    Omega4 = H1*B4*Sigma_sq*(H1*B4)';

    Phi1 = H1*B1*rho/(H1*B4);
    Phi2 = H1*B2*rho/(H1*B1);
    Phi3 = H1*B3*rho/(H1*B2);
    Phi4 = H1*B4*rho/(H1*B3);

    phi1 = H1*A1 + H1*B1*c - Phi1*H1*A4;
    phi2 = H1*A2 + H1*B2*c - Phi2*H1*A1;
    phi3 = H1*A3 + H1*B3*c - Phi3*H1*A2;
    phi4 = H1*A4 + H1*B4*c - Phi4*H1*A3;

    Gamma1 = (H2*B1)/(H1*B1);
    Gamma2 = (H2*B2)/(H1*B2);
    Gamma3 = (H2*B3)/(H1*B3);
    Gamma4 = (H2*B4)/(H1*B4);

    gamma1 = H2*A1 - Gamma1*H1*A1;
    gamma2 = H2*A2 - Gamma2*H1*A2;
    gamma3 = H2*A3 - Gamma3*H1*A3;
    gamma4 = H2*A4 - Gamma4*H1*A4;
end
