function [penalty,Omega,gamma,Gamma,alpha,beta_transpose,cQ,rhoQ,beta_0]...
    = papa_str2redu_seasonal_xi_invariant(xi,alpha_0,Sigma,H1,H2)
% papa_str2redu_seasonal imposes cross section restrictions and converts 
% structure parameters into reduced form parameters for the seasonal model
% in HW4
%
% INPUTS:   xi,Sigma, alpha_0, c, rho: structural parameters. NOTE, xi,Sigma,
%               alpha_0 ARE 3d matrices and para(:,:,j) = para_j
% OUTPUTS:  penalty: the penalty function, is non-zero, if the resulting
%               matrices are not well-behaved
%           phi,Phi,Omega,gamma,Gamma: resulting reduced form parameters.
%               All of them are in 3d matrices, and para(:,:,j) = para_j

N = 2; J = 4;


%=========================create sigma*sigma'==============================
for j = 1:J
    Sigma_sq(:,:,j) = Sigma(:,:,j)*Sigma(:,:,j)';
     %eval(['Sigma' num2str(j) '_sq' ' = Sigma.j' num2str(j) '*Sigma.j' num2str(j) ';'])
end

%====================CALCULATE rhoQ_j, beta_0==============================
K_xi = [xi.^0,xi.^4,xi.^8];
rhoQ = ((K_xi*H1')\diag(xi)*(K_xi*H1'))';
beta_0 = (K_xi*H1')\ones(2,1);

%==========================calculate beta==================================
beta_transpose = zeros(12,2);  % ith row of beta_transpose is beta_i-1'
beta_transpose(1,:) = beta_0';
for i = 2:12
    beta_transpose(i,:) = beta_transpose(i-1,:)*rhoQ;
end

%{
G = zeros(J);
G(1,2) = 1; G(2,3) = 1; G(3,4) = 1; G(4,1) = 1;
%=============================create zeta==================================
zeta_0 = zeros(4,8);
zeta_4 = zeros(4,8);
zeta_8 = zeros(4,8);

for i = 1:J
    zeta_4 = zeta_4 + G^(i-1)*blkdiag(beta_transpose(5-i,:),beta_transpose(5-i,:),beta_transpose(5-i,:),beta_transpose(5-i,:));
end

for i = 1:8
    zeta_8 = zeta_8 + G^(i-1)*blkdiag(beta_transpose(9-i,:),beta_transpose(9-i,:),beta_transpose(9-i,:),beta_transpose(9-i,:));
end

%============================create psi====================================
alpha_0_tilda = [alpha_0(:,:);alpha_0(:,:);alpha_0(:,:);alpha_0(:,:)];

psi_0 = alpha_0_tilda;
psi_4 = G^4*alpha_0_tilda;
psi_8 = G^8*alpha_0_tilda;

for i = 1:4
    psi_4 = psi_4 + G^(i-1)/2*[beta_transpose(5-i,:)*Sigma_sq(:,:,1)*beta_transpose(5-i,:)';...
        beta_transpose(5-i,:)*Sigma_sq(:,:,2)*beta_transpose(5-i,:)';beta_transpose(5-i,:)*Sigma_sq(:,:,3)*beta_transpose(5-i,:)';...
        beta_transpose(5-i,:)*Sigma_sq(:,:,4)*beta_transpose(5-i,:)'];
end

for i = 1:8
    psi_8 = psi_8 + G^(i-1)/2*[beta_transpose(9-i,:)*Sigma_sq(:,:,1)*beta_transpose(9-i,:)';...
        beta_transpose(9-i,:)*Sigma_sq(:,:,2)*beta_transpose(9-i,:)';beta_transpose(9-i,:)*Sigma_sq(:,:,3)*beta_transpose(9-i,:)';...
        beta_transpose(9-i,:)*Sigma_sq(:,:,4)*beta_transpose(9-i,:)'];
end

%============================calculate cQ==================================
cQ_tilda = -(kron(H1,eye(J))*[zeta_0;zeta_4;zeta_8])\...
    (kron(H1,eye(J))*[psi_0;psi_4;psi_8]);

for j = 1:J
    cQ(:,:,j) = cQ_tilda((j-1)*2+1:(j-1)*2+2,:);
end
%}

%============================calculate cQ==================================
psi_0 = alpha_0;
psi_4 = alpha_0;
psi_8 = alpha_0;

for n = 1:4
    psi_4 = psi_4 + 1/2*beta_transpose(n,:)*Sigma_sq(:,:,jplusi (12, -n))*beta_transpose(n,:)';
end

for n = 1:8
    psi_8 = psi_8 + 1/2*beta_transpose(n,:)*Sigma_sq(:,:,jplusi (12, -n))*beta_transpose(n,:)';
end

cQ = -(H1*[zeros(1,2);sum(beta_transpose(1:4,:));sum(beta_transpose(1:8,:))])\...
    (H1*[psi_0;psi_4;psi_8]);

%==========================calculate alpha=================================
%{
alpha = zeros(12,1,J);
for j = 1:J
    alpha(1,:,j) = alpha_0(:,:);
end

for i = 2:12
    for j = 1:J
        alpha(i,:,j) = alpha(i-1,:,jplusi(j,1))+beta_transpose(i-1,:)*cQ(:,:,j) + 0.5 * beta_transpose(i-1,:)*Sigma_sq(:,:,j)*beta_transpose(i-1,:)';
    end
end
%}

alpha = zeros(12,1);
alpha(1) = alpha_0;
for i = 2:12
    %alpha(i) = alpha(i-1)+beta_transpose(i-1,:)*cQ + 0.5 * beta_sigsig_beta(i-1);
    alpha(i) = alpha(i-1)+beta_transpose(i-1,:)*cQ + 0.5 * beta_transpose(i-1,:)*Sigma_sq(:,:,jplusi (12, -i+1))*beta_transpose(i-1,:)';
end
%==================convert to reduced form parameters======================
Omega = zeros(N,N,J);
Gamma = zeros(1,N,J);
gamma = zeros(1,1,J);

for j = 1: J
    Bj = [beta_transpose(4-j+1,:);beta_transpose(8-j+1,:);beta_transpose(12-j+1,:)];
    %Aj = [alpha(4-j+1,:,j);alpha(8-j+1,:,j);alpha(12-j+1,:,j)];
    Aj = [alpha(4-j+1,:);alpha(8-j+1,:);alpha(12-j+1,:)];

    if rcond(H1*Bj)<1e-15
        penalty = 1e50;
    else
        penalty = 0;
        Omega(:,:,j) = H1*Bj*Sigma_sq(:,:,jplusi(j,-1))*(H1*Bj)'; 
        Gamma(:,:,j) = (H2*Bj)/(H1*Bj);
        gamma(:,:,j) = H2*Aj - Gamma(:,:,j)*H1*Aj;
    end
end
